from rest_framework import serializers
from .models import Headshot, Post, Like, Comment, PostMedia, PostReport, ReportReason
from userprofile_app.models import Profile


class UserProfileSerializer(serializers.ModelSerializer):
    username = serializers.CharField(source='user.username')
    profile_image = serializers.FileField()
    full_name = serializers.CharField(source='user.full_name')

    class Meta:
        model = Profile
        fields = ['username', 'profile_image','full_name']


class PostSerializer(serializers.ModelSerializer):
    user = UserProfileSerializer(source='user.profile', read_only=True)
    likes_count = serializers.IntegerField(source='likes.count', read_only=True)
    comments_count = serializers.IntegerField(source='comments.count', read_only=True)
    post_id = serializers.IntegerField(source='id', read_only=True)

    class Meta:
        model = Post
        fields = ['post_id', 'user', 'caption', 'location', 'media', 'media_type', 'created_at', 'likes_count', 'comments_count']
        read_only_fields = ['user', 'media_type', 'created_at']

class LikeSerializer(serializers.ModelSerializer):
    user = UserProfileSerializer(source='user.profile', read_only=True)
    like_id = serializers.IntegerField(source='id', read_only=True) 

    class Meta:
        model = Like
        fields = ['user_id','like_id', 'user', 'created_at']

class CommentSerializer(serializers.ModelSerializer):
    user = UserProfileSerializer(source='user.profile', read_only=True)
    comment_id = serializers.IntegerField(source='id', read_only=True)
    replies = serializers.SerializerMethodField()
    total_likes = serializers.SerializerMethodField()

    class Meta:
        model = Comment
        fields = ['id', 'user_id','user','comment_id', 'post', 'parent', 'content', 'created_at', 'is_deleted', 'replies', 'total_likes']

    def get_replies(self, obj):
        replies = obj.replies.filter(is_deleted=False)
        current_user = self.context['request'].user  # Get the current user from context
        serialized_replies = []

        for reply in replies:
            reply_data = CommentSerializer(reply, context=self.context).data
            reply_data['comment_like_status'] = reply.user == current_user
            serialized_replies.append(reply_data)

        return serialized_replies

    def get_total_likes(self, obj):
        return obj.likes.count()

class HeadshotSerializer(serializers.ModelSerializer):
    user = UserProfileSerializer(source='user.profile', read_only=True)
    headshot_id = serializers.IntegerField(source='id', read_only=True)  

    class Meta:
        model = Headshot
        fields = ['headshot_id', 'user', 'banner', 'film_name', 'role_played', 'created_at']

class PostMediaSerializer(serializers.ModelSerializer):
    class Meta:
        model = PostMedia
        fields = ['id', 'media', 'media_type']
        
class PostSerializer(serializers.ModelSerializer):
    user = UserProfileSerializer(source='user.profile', read_only=True)
    media_files = PostMediaSerializer(many=True, read_only=True)
    # thumbnail = serializers.ImageField()

    class Meta:
        model = Post
        fields = ['id', 'user', 'caption', 'location', 'created_at', 'media_files']


class ReportReasonSerializer(serializers.ModelSerializer):
    class Meta:
        model = ReportReason
        fields = ['id', 'reason']

class PostReportSerializer(serializers.ModelSerializer):
    class Meta:
        model = PostReport
        fields = ['post', 'reason']